###########################################################################
#
# ExcelPlugin.pm -- plugin for importing Microsoft Excel files.
#  (currently only versions 95 and 97)
#
# A component of the Greenstone digital library software
# from the New Zealand Digital Library Project at the 
# University of Waikato, New Zealand.
#
# Copyright (C) 2002 New Zealand Digital Library Project
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
###########################################################################

package ExcelPlugin;

use ConvertBinaryFile;
use strict;
no strict 'refs'; # allow filehandles to be variables and viceversa

sub BEGIN {
    @ExcelPlugin::ISA = ('ConvertBinaryFile');
}

my $arguments = 
    [ { 'name' => "process_exp",
	'desc' => "{BasePlugin.process_exp}",
	'type' => "regexp",
	'reqd' => "no",
	'deft' => &get_default_process_exp() }
      ];

my $options = { 'name'     => "ExcelPlugin",
		'desc'     => "{ExcelPlugin.desc}",
		'abstract' => "no",
		'inherits' => "yes",
		'srcreplaceable' => "yes", # Source docs in Excel format can be replaced with GS-generated html
		'args'     => $arguments };

sub new {
    my ($class) = shift (@_);
    my ($pluginlist,$inputargs,$hashArgOptLists) = @_;
    push(@$pluginlist, $class);

    push(@{$hashArgOptLists->{"ArgList"}},@{$arguments});
    push(@{$hashArgOptLists->{"OptList"}},$options);
    
    my $self = new ConvertBinaryFile($pluginlist, $inputargs, $hashArgOptLists);

    if ($self->{'info_only'}) {
	# don't worry about any options etc
	return bless $self, $class;
    }

    $self->{'filename_extension'} = "xls";
    $self->{'file_type'} = "Excel";

    my $secondary_plugin_options = $self->{'secondary_plugin_options'};
    if (!defined $secondary_plugin_options->{'HTMLPlugin'}) {
	$secondary_plugin_options->{'HTMLPlugin'} = [];
    }
    if (!defined $secondary_plugin_options->{'TextPlugin'}) {
	$secondary_plugin_options->{'TextPlugin'} = [];
    }
    my $html_options = $secondary_plugin_options->{'HTMLPlugin'};
    my $text_options = $secondary_plugin_options->{'TextPlugin'};

    push(@$html_options, "-input_encoding", "utf8");
    push(@$html_options,"-extract_language") if $self->{'extract_language'};
    push(@$html_options, "-file_rename_method", "none");
    
    push(@$text_options, "-input_encoding", "utf8");
    push(@$text_options,"-extract_language") if $self->{'extract_language'};
    push(@$text_options, "-file_rename_method", "none");

    $self = bless $self, $class;
    
    $self->load_secondary_plugins($class,$secondary_plugin_options,$hashArgOptLists);
    return bless $self;    
}

sub convert_post_process_old
{
    my $self = shift (@_);
    my ($conv_filename) = @_;
    
    my $outhandle=$self->{'outhandle'};
    
    my ($language, $encoding) = $self->textcat_get_language_encoding ($conv_filename);
    
    # read in file ($text will be in utf8)
    my $text = "";
    $self->read_file ($conv_filename, $encoding, $language, \$text);
    
    # turn any high bytes that aren't valid utf-8 into utf-8.
    #unicode::ensure_utf8(\$text);
    
    # Write it out again!
    #$self->utf8_write_file (\$text, $conv_filename);
}

sub get_default_process_exp {
    my $self = shift (@_);
    return q^(?i)\.xls$^;
}
    

1;
